package test.forum.json;
import java.io.Serializable;
import java.net.URI;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlSeeAlso;
import javax.xml.bind.annotation.XmlType;

@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name = OprResourceBase.XML_NAME)
@XmlType(name = OprResourceBase.XML_TYPE,
         propOrder = {"self", "selfType", "version", "id", "instanceVersion"})
public abstract class OprResourceBase implements OprAny, Serializable
{
  private static final long serialVersionUID = 8464967716384069995L;

  public static final String COMPONENT = "opr";
  public static final String BASE_URN = "urn:x-hp:2009:software:data_model:" + COMPONENT + ":";
  public static final String BASE_TYPE = BASE_URN + "type:";
  public static final String BASE_REF_TYPE = BASE_TYPE + "reference:";
  public static final String BASE_ROLE = BASE_URN + "relationship:";
  public static final String BASE_TAXONOMY = BASE_URN + "taxonomy:";

  public static final String NAMESPACE_PREFIX = "";
  public static final String NAMESPACE = "http://www.hp.com/2009/software/opr/data_model";
  public static final String XML_SCHEMA_NAMESPACE_PREFIX = "xs";
  public static final String XML_SCHEMA_NAMESPACE = "http://www.w3.org/2001/XMLSchema";
  public static final String XML_SCHEMA_INSTANCE_NAMESPACE_PREFIX = "xsi";
  public static final String XML_SCHEMA_INSTANCE_NAMESPACE = "http://www.w3.org/2001/XMLSchema-instance";
  public static final String VERSION = "1.0";

  public static final String XML_NAME = "resource_base";
  public static final String XML_TYPE = XML_NAME + "_type";
  public static final String TYPE = BASE_TYPE + XML_NAME;
  public static final URI TYPE_URI = URI.create(TYPE);

  private URI m_self;
  private URI m_selfType;
  private String m_version;
  private Integer m_instanceVersion;
  private String m_id;

  public OprResourceBase()
  {
    this(TYPE_URI, VERSION);
  }

  protected OprResourceBase(final URI selfType, final String version)
  {
    m_selfType = selfType;
    m_version = version;
  }

  @XmlElement(name = "id")
  public String getId()
  {
    return m_id;
  }

  public void setId(String id)
  {
    m_id = id;
  }

  @XmlAttribute(name = "self")
  public URI getSelf()
  {
    return m_self;
  }

  public void setSelf(final URI self)
  {
    m_self = self;
  }

  //@XmlAttribute(name = "selfType")
  @XmlAttribute(name = "type")
  public URI getSelfType()
  {
    return m_selfType;
  }

  public void setSelfType(final URI selfType)
  {
    m_selfType = selfType;
  }

  @XmlAttribute(name = "version")
  public String getVersion()
  {
    return m_version;
  }

  public void setVersion(final String version)
  {
    m_version = version;
  }

  @XmlElement(name = "version")
  public Integer getInstanceVersion()
  {
    return m_instanceVersion;
  }

  public void setInstanceVersion(final Integer instanceVersion)
  {
    m_instanceVersion = instanceVersion;
  }

  @Override
  public int hashCode()
  {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((m_id == null) ? 0 : m_id.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj)
  {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (this.getClass() != obj.getClass())
      return false;

    OprResourceBase other = (OprResourceBase) obj;
    if (m_id == null)
    {
      if (other.m_id != null)
        return false;
    }
    else if (!m_id.equals(other.m_id))
      return false;

    return true;
  }
}